const chalk = require("chalk");
const path = require("path");
const fs = require("fs");
const cp = require("child_process");
const { Confirm } = require("enquirer");

const prompt = new Confirm({
    name: "question",
    message: "It seems like the certificate required for https is missing. Generate now?",
    initial: true
});

const APP_DATA =
    process.env.APPDATA ||
    (process.platform == "darwin" ? process.env.HOME + "/Library/Preferences" : process.env.HOME + "/.local/share");

async function generateCertificate() {
    return new Promise((resolve, reject) => {
        var child = cp.spawn(
            "sudo",
            [
                "openssl",
                "req",
                "-x509",
                "-nodes",
                "-days",
                "3650",
                "-newkey",
                "rsa:2048",
                "-keyout",
                path.resolve(APP_DATA, "flex-ui-dev-proxy/flex-ui-dev-proxy.key"),
                "-out",
                path.resolve(APP_DATA, "flex-ui-dev-proxy/flex-ui-dev-proxy.crt")
            ],
            { stdio: ["inherit", "inherit", "inherit"] }
        );

        child.on("exit", (retVal) => (retVal ? reject() : resolve()));
        child.on("error", () => reject());
    });
}

async function ownCertificate() {
    return new Promise((resolve, reject) => {
        var child = cp.spawn(
            "sudo",
            [
                "chown",
                process.env.USER,
                path.resolve(APP_DATA, "flex-ui-dev-proxy/flex-ui-dev-proxy.key"),
                path.resolve(APP_DATA, "flex-ui-dev-proxy/flex-ui-dev-proxy.crt")
            ],
            { stdio: ["inherit", "inherit", "inherit"] }
        );

        child.on("exit", (retVal) => (retVal ? reject() : resolve()));
        child.on("error", () => reject());
    });
}

async function getCredentials(host, port) {
    const crtPath = path.resolve(APP_DATA, "flex-ui-dev-proxy/flex-ui-dev-proxy.crt");
    const keyPath = path.resolve(APP_DATA, "flex-ui-dev-proxy/flex-ui-dev-proxy.key");

    try {
        fs.readFileSync(crtPath, "utf8");
    } catch (e) {
        fs.mkdirSync(path.resolve(APP_DATA, "flex-ui-dev-proxy"), { recursive: true });

        prompt.start();

        const answer = await prompt.run();
        if (!!answer) {
            console.log(
                chalk.cyan("\nGenerating the certificate. Please enter your sudo password and at least one field:\n")
            );
            try {
                await generateCertificate();
                await ownCertificate();
            } catch (e) {
                console.log(
                    chalk.red(
                        "\n\nError:\n\nThere was an error when generating your certificate. Check the console output above.\n\n"
                    )
                );
                process.exit(1);
            }
            console.log(chalk.cyan(`\n\nCertificate generation done.\n\n`));

            console.log(
                chalk.red("Important!"),
                chalk.yellow(`\n\nYou have just generated a self-signed certificate, that browsers like Chrome don't consider to be trustworthy.

Therefore, when you open https://${host}:${port}, you will land on the NET::ERR_CERT_INVALID error page.

To let Chrome know you trust the certificate on ${host}, type "thisisunsafe" on that error page and wait for the page to load.

Without this, the proxy won't work properly.

For more information, please visit https://www.google.com/search?q=ERR_CERT_INVALID+localhost+self-signed+certificate
    `)
            );
        } else {
            console.log(
                chalk.red("Cannot start https without a certificate."),
                `\nTo use your own, please provide:\n${crtPath}\n${keyPath}`
            );
            console.log("Exiting.");
            process.exit(0);
        }
    }

    try {
        const key = fs.readFileSync(keyPath, "utf8");
        const cert = fs.readFileSync(crtPath, "utf8");
        console.log(`Using certificate ${crtPath}`);
        return { key, cert };
    } catch (e) {
        console.log(chalk.red("There was an error reading the certificate file.", e));
        process.exit(1);
    }
}

module.exports = { getCredentials };
